<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Shane Caraveo <Shane@Caraveo.com>                           |
// +----------------------------------------------------------------------+
//
// $Id: smtp.php 322284 2012-01-14 13:15:06Z clockwerx $
//
// include soap client class
include("SOAP/Client.php");

$soapclient = new SOAP_Client("mailto:shane@caraveo.com");
$options = array('namespace' => 'http://soapinterop.org/', 'from' => 'shane@caraveo.com');
$return = $soapclient->call("echoString", array("inputString" => "this is a test"), $options);
print_r($return);
unset($soapclient);
