<?php
/**
 * TCP client.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 2.02 of the PHP license,
 * that is bundled with this package in the file LICENSE, and is available at
 * through the world-wide-web at http://www.php.net/license/2_02.txt.  If you
 * did not receive a copy of the PHP license and are unable to obtain it
 * through the world-wide-web, please send a note to license@php.net so we can
 * mail you a copy immediately.
 *
 * @category   Web Services
 * @package    SOAP
 * @author     Shane Caraveo <Shane@Caraveo.com>   Port to PEAR and more
 * @author     Jan Schneider <jan@horde.org>       Maintenance
 * @copyright  2003-2007 The PHP Group
 * @license    http://www.php.net/license/2_02.txt  PHP License 2.02
 * @link       http://pear.php.net/package/SOAP
 */

require_once 'SOAP/Client.php';

/* Client. */
$soapclient = new SOAP_Client('tcp://127.0.0.1:82');

/* Namespace. */
$options = array('namespace' => 'urn:SOAP_Example_Server', 'trace' => true);

/* One. */
$params = array('string' => 'this is string 1');
$ret1 = $soapclient->call('echoString', $params, $options);
echo "WIRE: \n" . $soapclient->getWire();
print_r($ret1);

/* Two. */
$params = array('string' => 'this is string 2');
$ret2 = $soapclient->call('echoString', $params, $options);
echo "WIRE: \n" . $soapclient->getWire();
print_r($ret2);
